/***************************************************************************
 *   $Id: parse.h,v 1.1.1.1 2011-11-17 16:42:28 scg Exp $
 *                                                                         *
 *   (C)2009 St(u)dio of Computer Games                                    *
 *   Alexander Ozumenko <a.ozumenko@gmail.com>                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __PARSE_H_
#define __PARSE_H_



#include <sys/types.h>
#include <limits.h>



enum parse_state {
    STATE_DATA,
    STATE_TMPL_CLOSE,
    STATE_TMPL_0,
    STATE_TMPL_1,
    STATE_TMPL_2,
    STATE_TMPL_3,
    STATE_TMPL_4,
    STATE_TMPL_5,

    STATE_VAR_0,
    STATE_VAR_1,

    STATE_INCLUDE_IF,
    STATE_INCLUDE_0,
    STATE_INCLUDE_1,
    STATE_INCLUDE_2,
    STATE_INCLUDE_3,
    STATE_INCLUDE_4,

    STATE_LOOP_0,
    STATE_LOOP_1,
    STATE_LOOP_2,

    STATE_UNLESS_0,
    STATE_UNLESS_1,
    STATE_UNLESS_2,
    STATE_UNLESS_3,
    STATE_UNLESS_4,

    STATE_ELSE_0,
    STATE_ELSE_1,
    STATE_ELSE_2,

    STATE_NAME_SPACE,

    STATE_NAME_0,
    STATE_NAME_1,
    STATE_NAME_2,
    STATE_NAME_3,
    STATE_NAME_VAL,
    STATE_NAME_VAL_QUOT,
    STATE_NAME_VAL_NOQUOT,

    STATE_TAG_END
};


enum parse_tag {
    TAG_NONE = -1,
    TAG_VAR = 0,
    TAG_LOOP,
    TAG_LOOP_CLOSE,
    TAG_INCLUDE,
    TAG_IF,
    TAG_IF_CLOSE,
    TAG_UNLESS,
    TAG_UNLESS_CLOSE,
    TAG_ELSE
};


enum parse_tag_type {
    TAG_OPEN,
    TAG_CLOSE
};



struct parse;

typedef int parse_tag_hook_t(struct parse *parse, enum parse_tag tag, const char *var);
typedef int parse_data_hook_t(struct parse *parse, const char *data, size_t len);



struct parse {
    enum parse_state state;
    enum parse_tag tag;
    enum parse_tag_type tag_type;

    /* tag buffer */
    char *buf;
    size_t buf_size;
    off_t buf_pos;
    off_t name_start;
    size_t name_length;

    /* statistic */
    int line, col, tag_line, tag_col;

    char work_dir[PATH_MAX + 1];
    void *user_data;
    parse_tag_hook_t *tag_hook;
    parse_data_hook_t *data_hook;
};



/* init parse structure */
void parse_init(struct parse *parse, void *user_data,
            parse_tag_hook_t *tag_hook, parse_data_hook_t *data_hook);

int parse_make_work_dir(struct parse *parse, const char *filename);

/* release parse structure */
int parse_release(struct parse *parse);

/* parse template chunk */
int parse_routine(struct parse *parse, const char *data, int length);

/* get current TAG position (must be used by TAG hook) */
int parse_tag_col(struct parse *parse);
int parse_tag_row(struct parse *parse);



#endif
